# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from gtk import TreeView, TreeStore, HBox, ToolButton, STOCK_STOP, Toolbar, \
		ORIENTATION_VERTICAL, ScrolledWindow, POLICY_AUTOMATIC, TreeViewColumn, \
		CellRendererText, TOOLBAR_ICONS, ICON_SIZE_SMALL_TOOLBAR, CellRendererPixbuf, \
		image_new_from_stock, STOCK_EXECUTE, ICON_SIZE_MENU, TreeViewColumn, CellRendererText
from gtk.gdk import Pixbuf, pixbuf_new_from_file
from gobject import SIGNAL_RUN_LAST, TYPE_NONE, TYPE_INT, TYPE_STRING

from LaTeXPlugin.Logger import debug
from Message import ERROR, WARNING
from LaTeXPlugin.Installation import Installation


class BuildView(HBox):
	
	__gsignals__ = {
		"line-selected" : (SIGNAL_RUN_LAST, TYPE_NONE, ([TYPE_STRING, TYPE_INT])),	# filename, line
		"abort-clicked" : (SIGNAL_RUN_LAST, TYPE_NONE, ([]))
	}
	
	def __init__(self):
		HBox.__init__(self)
		
		self.__pixbuf_running = pixbuf_new_from_file(Installation().path + "/pixmaps/run.png")
		self.__pixbuf_failed = pixbuf_new_from_file(Installation().path + "/pixmaps/error.png")
		self.__pixbuf_succeeded = pixbuf_new_from_file(Installation().path + "/pixmaps/okay.png")
		self.__pixbuf_aborted = pixbuf_new_from_file(Installation().path + "/pixmaps/abort.png")
		self.__pixbuf_warning = pixbuf_new_from_file(Installation().path + "/pixmaps/warning.png")
		self.__pixbuf_error = pixbuf_new_from_file(Installation().path + "/pixmaps/error.png")
		
		scroll = ScrolledWindow()
		scroll.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
		scroll.add(self.__get_treeview())
		
		self.pack_start(scroll, True)
		self.pack_start(self.__get_toolbar(), False)
		
		self.show_all()
		
	def __get_treeview(self):
		self.__model = TreeStore(Pixbuf, str, object, str, str)		# icon, markup, Message object, line, file
		
		self.__view = TreeView(self.__model)
		#self.__view.set_headers_visible(False)
		
		column = TreeViewColumn("Job")

		pixbuf_renderer = CellRendererPixbuf()
		column.pack_start(pixbuf_renderer, False)
		column.add_attribute(pixbuf_renderer, "pixbuf", 0)
		
		text_renderer = CellRendererText()
		column.pack_start(text_renderer, True)
		column.add_attribute(text_renderer, "markup", 1)
		
		self.__view.append_column(column)
		self.__view.append_column(TreeViewColumn("Line", CellRendererText(), text=3))
		self.__view.append_column(TreeViewColumn("File", CellRendererText(), text=4))
		
		self.__view.connect("cursor-changed", self.__cursor_changed)
		
		return self.__view
	
	def __get_toolbar(self):
		#
		# TODO: tooltips
		#
		self.cancel_button = ToolButton(STOCK_STOP)
		self.cancel_button.set_sensitive(False)
		self.cancel_button.connect("clicked", lambda x: self.emit("abort-clicked"))
		
		self.__toolbar = Toolbar()
		self.__toolbar.set_style(TOOLBAR_ICONS)
		#self.__toolbar.set_icon_size(ICON_SIZE_SMALL_TOOLBAR)		# deprecated???
		self.__toolbar.set_orientation(ORIENTATION_VERTICAL)
		self.__toolbar.insert(self.cancel_button, -1)
		
		return self.__toolbar
	
	def __cursor_changed(self, view):
		store, it = view.get_selection().get_selected()
		if not it:
			return
			
		message = store.get_value(it, 2)
		
		if message:
			if message.line_from:
				line = int(message.line_from)
				self.emit("line-selected", message.filename, line)
	
	def set_profile(self, profile):
		"""
		Set a build profile to display
		"""
		self.__model.clear()
		root = self.__model.append(None, [None, "<b>%s</b>" % profile.name, None, "", ""])
		for job in profile.jobs:
			self.__model.append(root, [None, "<tt>%s</tt>" % job.get_command(), None, "", ""])
		self.__view.expand_all()
	
	def set_job_started(self, index):
		debug(self, "set_job_started", str(index))
		
		self.cancel_button.set_sensitive(True)
		self.__model[(0, index)][0] = self.__pixbuf_running
	
	def set_job_finished(self, index):
		"""
		Mark a job as finished
		"""
		self.cancel_button.set_sensitive(False)
		self.__model[(0, index)][0] = self.__pixbuf_succeeded
		
	def set_job_aborted(self, index):
		self.cancel_button.set_sensitive(False)
		self.__model[(0, index)][0] = self.__pixbuf_aborted
	
	def set_job_failed(self, index):
		"""
		A job has failed
		
		Mark this job and display the messages below it
		"""
		#
		# TODO: scroll to bottom
		#
		self.cancel_button.set_sensitive(False)
		
		self.__model[(0, index)][0] = self.__pixbuf_failed

	def append_messages(self, index, messages):
		parent = self.__model.get_iter_from_string("0:%s" % index)
		
		for message in messages:
			if message.severity == ERROR:
				pb = self.__pixbuf_error
			elif message.severity == WARNING:
				pb = self.__pixbuf_warning
			
			line = ""
			if message.line_from:
				line = "%s" % (message.line_from + 1)
				if message.line_to:
					line += "-%s" % (message.line_to + 1)
			
			self.__model.append(parent, [pb, message.text, message, line, message.get_basename()])
			
		self.__view.expand_all()
	
	def set_summary(self, index, summary):
		self.__model[(0, index)][1] += " [%s]" % summary
		
		