# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from re import compile
from os.path import dirname, basename, splitext
from glob import glob

from LaTeXPlugin.Logger import debug, error
from LaTeXPlugin.latex.LatexParser import LatexParser
from LaTeXPlugin.outline import OutlineView
from LaTeXPlugin.autocomplete.Controller import Controller
from LaTeXPlugin.common import TEXT, MATH, ConditionalSource, PrefaceSource
from LaTeXPlugin.builder.Message import ERROR, WARNING
from LaTeXPlugin.bibtex.BibTeXFile import BibTeXFile

ERROR_COLOR = "#ffe6e6"
WARNING_COLOR = "#fff2cc"

_pattern_indent = compile("[ \t]+")

class LatexLanguageHelper(object):
	def __init__(self, document_helper):
		debug(self, "__init__")
		
		self.__document_helper = document_helper
		self.__document_helper_manager = self.__document_helper.document_helper_manager
		self.__window_helper = self.__document_helper_manager.window_helper
		
		self.__view = self.__document_helper.view
		self.__buffer = self.__view.get_buffer()
		
		self.__parser = LatexParser()
		
		try:
			self.__buffer.create_tag("error", background=ERROR_COLOR)
			self.__buffer.create_tag("warning", background=WARNING_COLOR)
		except TypeError, s:
			# FIXME: this happens when deactivating and reactivating the plugin
			error(self, "__init__", s)
		
		self.__outline_view = OutlineView(self.__parser.outline_model)
		
		self.__outline_view.connect("node-range-selected", self.__outline_node_range_selected)
		self.__outline_view.connect("file-activated", self.__outline_file_activated)
		
		self.__autocomplete_controller = Controller(self, self.__view, \
													self.__window_helper.autocomplete_popup, \
													self.__window_helper.hint_popup, \
													self.__window_helper.settings)
		
		self.__document_helper.connect("source-changed", self.__source_changed)
		self.__document_helper.connect("document-changed", self.__source_changed)
		
		self.__synchronize()
	
	def __synchronize(self):
		debug(self, "__synchronize")
		
		self.__parser.parse(self.__document_helper.text, self.__document_helper.cursor_offset, True)
		
		self.__autocomplete_controller.set_open_environment(self.__parser.open_environment)
		self.__autocomplete_controller.set_labels(self.__parser.labels)
		self.__autocomplete_controller.set_newcommands(self.__parser.newcommands)
		
		self.__outline_view.expand_all()
		
		filename = self.__document_helper.filename
		
		##################
		# bibtex
		
		# get local bibitems
		entries = self.__parser.bibitems
		#entries = []
		
		# get entries from referenced bibtex file
		if filename:
			dir = dirname(filename)
			
			debug(self, "__synchronize", "Referenced BibTeX files: " + str(self.__parser.bibliography_files))
			
			for filename in self.__parser.bibliography_files:
				if filename[0] != "/":		# TODO: can there be absolute paths in that argument???
					filename = "%s/%s.bib" % (dir, filename)
				
				try:
					model = self.__document_helper_manager.bibtex_model_factory.get_model(filename)
					
					debug(self, "", "Adding BibTeX entries: " + str(model.entries))
					
					entries.extend(model.entries)
				except Exception, e:
					error(self, "__synchronize", "Getting model for %s failed: %s" % (filename, e))
		
		# pass entries to completion
		self.__autocomplete_controller.set_bibitems(entries)
		
		###################
		# ambient files
		
		if filename:
			# .tex
			
			# TODO: exclude this file
			
			files = [splitext(basename(f))[0] for f in glob("%s/*.tex" % dirname(filename))]
			files.sort()
			
			self.__autocomplete_controller.set_ambient_tex_files(files)
			
			# .bib
			
			# TODO: exclude already referenced files
			
			files = [splitext(basename(f))[0] for f in glob("%s/*.bib" % dirname(filename))]
			files.sort()
			
			self.__autocomplete_controller.set_ambient_bib_files(files)
			
			# image files
			
			# TODO: improve and take extensions from \DeclareGraphicsRule
			
			files = []
			for suffix in ["ps","pdf","png","jpg","eps"]:
				files.extend([basename(f) for f in glob("%s/*.%s" % (dirname(filename), suffix))])
			files.sort()
			
			self.__autocomplete_controller.set_ambient_image_files(files)
	
	def __source_changed(self, document_helper):
		debug(self, "__source_changed")
		self.reset_highlight()
		self.__synchronize()
	
	def __outline_node_range_selected(self, outline_view, start, end):
		self.__document_helper.select_range(start, end)
	
	def __outline_file_activated(self, outline_view, filename):
		if filename[0] != "/":
			dir = dirname(self.__document_helper.get_filename())
			filename = "%s/%s" % (dir, filename)
		
		self.__document_helper_manager.set_active_helper(filename)
	
	@property
	def outline_view(self):
		return self.__outline_view
	
	def get_indent(self, line=None):
		"""
		Return the indentation string of the line at the cursor or at a specific
		line number
		"""
		if line:
			i_start = self.__buffer.get_iter_at_line(line)
		else:
			i_start = self.__buffer.get_iter_at_mark(self.__buffer.get_insert())
			i_start.set_line_offset(0)
		
		i_end = i_start.copy()
		i_end.forward_to_line_end()
		string = self.__buffer.get_text(i_start, i_end)
		
		match = _pattern_indent.match(string)
		if match:
			return match.group()
		else:
			return ""
	
	def insert_source(self, source):
		""" 
		Inserts a source depending on its type 
		"""
		
		debug(self, "insert_source", str(source))
		
		if type(source) is ConditionalSource:
			for p in source.get_required_packages():
				if not self.__contains_package(p):
					self.insert_source(PrefaceSource("\\usepackage{%s}\n" % p))
			self.insert_source(source.get_source())
			
		elif type(source) is PrefaceSource:
			# TODO: improve the way \usepackageS are inserted 
			line = self.__find_begin_document()
			if line > 0:
				line -= 1
			self.__insert_string(source.get_source(), line)
			
		elif type(source) is dict:
			try:
				self.insert_source(source[self.context])
				debug(self, "insert_source", "type is dict")
			except KeyError:
				# FIXME: work-around for the bad context recognition of the current LaTeX parser
				self.insert_source(source.values()[0])

				self.__window_helper.push_status_message("This LaTeX command is not defined in the current environment")
				
		elif type(source) is str or type(source) is unicode:
			self.__insert_string(source)
			
		elif type(source) is tuple or type(source) is list:
			debug(self, "insert_source", "type is list")
			self.__insert_pair(source)
			
		else:
			error(self, "insert_source", "unknown source type: %s" % type(source))
	
	def __find_begin_document(self):
		i = self.__buffer.get_start_iter()
		try:
			start, end = i.forward_search("\\begin{document}", 0, self.__buffer.get_end_iter())
			return start.get_line()
		except TypeError:	# unpack non-sequence -> no "\begin{document}"
			return -1
	
	def __contains_package(self, p):
		# TODO: check parsed symbols
		pattern = compile(r"\\usepackage(\[\w*?\])?\{" + p + "\}")
		return bool(pattern.search(self.__document_helper.text))
	
	def __insert_string(self, s, line=None):
		if line:
			indent = self.get_indent(line)
			s = "%s%s" % (indent, s)			# TODO: follow indent on multiline inserts
			self.__buffer.insert(self.__buffer.get_iter_at_line(line), s)
		else:
			indent = self.get_indent()
			self.__buffer.insert_at_cursor(s.replace("\n", "\n" + indent))
			
		self.__view.grab_focus()
		
	def __insert_pair(self, pair):
		"""
		Insert two strings and place the cursor between them
		"""
		bounds = self.__buffer.get_selection_bounds()
		if len(bounds):
			# store the selection marks
			m_left = self.__buffer.create_mark(None, bounds[0], False)
			m_right = self.__buffer.create_mark(None, bounds[1], True)
			self.__buffer.insert(bounds[0], pair[0])				# insert first piece at the beginning...
			i_right = self.__buffer.get_iter_at_mark(m_right)
			self.__buffer.insert(i_right, pair[1])					# ...and the other one at the end
			# restore selection
			i_left = self.__buffer.get_iter_at_mark(m_left)
			i_right = self.__buffer.get_iter_at_mark(m_right)
			self.__buffer.select_range(i_left, i_right)
			# delete marks
			self.__buffer.delete_mark(m_left)
			self.__buffer.delete_mark(m_right)
		else:
			indent = self.get_indent()
			# insert first item
			left = pair[0].replace("\n", "\n" + indent)
			self.__buffer.insert_at_cursor(left)
			# copy the insert mark
			mark = self.__buffer.create_mark(None, self.__buffer.get_iter_at_mark(self.__buffer.get_insert()), True)
			# insert the remaining pieces
			right = pair[1].replace("\n", "\n" + indent)
			self.__buffer.insert_at_cursor(right)
			self.__buffer.place_cursor(self.__buffer.get_iter_at_mark(mark))	# place cursor after the first one
			self.__buffer.delete_mark(mark)										# delete mark
	
	def get_text_after_cursor(self, n):
		"""
		Return the first n characters after the cursor 
		"""
		iter_l = self.__buffer.get_iter_at_mark(self.__buffer.get_insert())
		
		iter_r = iter_l.copy()
		iter_r.forward_chars(n)
		
		return iter_l.get_text(iter_r)
	
	def move_cursor(self, n):
		"""
		Move the cursor by n characters (n != 0)
		"""
		if n == 0:
			return
		
		iter = self.__buffer.get_iter_at_mark(self.__buffer.get_insert())
		
		if n < 0:
			iter.backward_chars(abs(n))
		elif n > 0:
			iter.forward_chars(n)
			
		self.__buffer.place_cursor(iter)
	
	@property
	def context(self):
		"""
		Try to guess the current context
		"""
		text = self.__document_helper.text
		
		counter = 0
		for item in [r"$", r"{math", r"{displaymath", r"{eqnarray", r"\[", r"\]", r"{align", r"{equation"]:
			counter += text.count(item)
			
		if counter % 2:
			env = MATH
		else:
			env = TEXT
		
		return env
	
	def highlight_line(self, line, severity=ERROR):
		"""
		Highlight a line in the TextBuffer
		"""
		iter_l = self.__buffer.get_iter_at_line(line)
		
		iter_r = iter_l.copy()
		iter_r.forward_to_line_end()
		
		if severity == WARNING:
			self.__buffer.apply_tag_by_name("warning", iter_l, iter_r)
		else:
			self.__buffer.apply_tag_by_name("error", iter_l, iter_r)
	
	def reset_highlight(self):
		"""
		Undo all highlighting
		
		Called at source-changed and from the BuildEngine
		"""
		self.__buffer.remove_tag_by_name("error", self.__buffer.get_start_iter(), 
										self.__buffer.get_end_iter())
		self.__buffer.remove_tag_by_name("warning", self.__buffer.get_start_iter(),
										self.__buffer.get_end_iter())
	
	@property
	def is_master(self):
		my_filename = self.__document_helper.filename
		if not my_filename:
			debug(self, "is_master", "I have no filename")
			return False
		
		s = open(my_filename).read()
		
		return bool(s.find(r"\begin{document}") != -1)
	
	@property
	def master_filename(self):
		my_filename = self.__document_helper.filename
		if not my_filename:
			debug(self, "get_master_filename", "I have no filename")
			return None
		
		# check this document	
		if self.is_master:
			debug(self, "get_master_filename", "I don't have one, it's me")
			return my_filename
		
		# search the directory for my master
		from os.path import splitext, basename, dirname
		from glob import glob
		
		name = splitext(basename(my_filename))[0]
		for filename in glob(dirname(my_filename) + "/*.tex"):
			s = open(filename).read()
			if s.find(r"\include{" + name) != -1 or s.find(r"\input{" + name) != -1:	# this one includes the file
				if s.find(r"\begin{document}") != -1:	# and it's a full document: bingo!
					debug(self, "get_master_filename", "found master document: " + filename)
					return filename
				else:									# it's not a full document either: not found
					error(self, "get_master_filename", "my master " + filename + " isn't a full document either, this is not allowed!")
					return None
		error(self, "get_master_filename", "could not find my master")
		
		# no master found
		return None
	
	def _comment_selection(self, uncomment=False):
		"""
		"""
		bounds = self.__buffer.get_selection_bounds()
		
		if not bounds:
			return
		
		it_start, it_end = bounds
		
		# return to line start
		if it_start.get_line_offset() > 0:
			it_start.set_line_index(0)
			
		
		while it_start.compare(itEnd) < 0:
			it_start.forward_word_start()
			
			it_start.forward_line()
	
	def comment_selection(self, uncomment=False):
		"""
		Comments or uncomments the current selection
		"""
		#
		# TODO: rework this to have an intelligent (un)comment function
		#
		bounds = self.__buffer.get_selection_bounds()
		if bounds:
			itStart, itEnd = bounds
			end = self.__buffer.create_mark(None, itEnd, True)
			
			if itStart.get_line_offset() > 0:
				itStart.set_line_index(0)
			
			maxPre = 0
			if uncomment:
				i = 0
				for line in self.__buffer.get_text(itStart, itEnd, False).splitlines():
					j = 0
					for char in line:
						if char in ["%", " ", "\t"]:
							j += 1
						else:
							break
					if i > 0:
						if j < len(line) and line[j] not in ["%", " ", "\t"]:
							if maxPre > j:
								maxPre = j
					else:
						maxPre = j
					i += 1
				
			while itStart.compare(itEnd) < 0:
				cur = self.__buffer.create_mark(None, itStart, True)
				if uncomment:
					if maxPre:
						itEnd = itStart.copy()
						itEnd.forward_chars(maxPre)
						self.__buffer.delete(itStart, itEnd)
						
				else:
					self.__buffer.insert(itStart, "% ", 2)
					
				itStart = self.__buffer.get_iter_at_mark(cur)
				itEnd = self.__buffer.get_iter_at_mark(end)
				self.__buffer.delete_mark(cur)
				itStart.forward_line()
			
			self.__buffer.delete_mark(end)
		else:
			## no selection
			it = self.__buffer.get_iter_at_mark(self.__buffer.get_insert())
			it.set_line_offset(0)
			if uncomment:
				itEnd = self.__buffer.get_end_iter()
				string = self.__buffer.get_text(it, itEnd, False)
				
				i = 0
				for char in string:
					if char in ["%", " ", "\t"]:
						i += 1
					else:
						break
				
				if i > 0:
					itEnd = it.copy()
					itEnd.forward_chars(i)
					self.__buffer.delete(it, itEnd)
			else:
				self.__buffer.insert(it, "% ", 2)
