# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from re import compile, VERBOSE, MULTILINE
from gtk import TreeStore
from gtk.gdk import Pixbuf, pixbuf_new_from_file
from os.path import basename

from LaTeXPlugin.Logger import debug, error
from LaTeXPlugin.Installation import Installation
from LaTeXPlugin.autocomplete.CommandStore import *
from LaTeXPlugin.outline import OutlineNodeRange, FileReference

levels = {
	"part" 			:  0,
	"chapter" 		:  1,
	"section" 		:  2,
	"subsection" 	:  3,
	"subsubsection" :  4,
	"paragraph" 	:  5,
	"subparagraph" 	:  6
}

# TODO: icon registry

pb_part = pixbuf_new_from_file(Installation().path + "/pixmaps/tree_part.png")
pb_chapter = pixbuf_new_from_file(Installation().path + "/pixmaps/tree_chapter.png")
pb_section = pixbuf_new_from_file(Installation().path + "/pixmaps/tree_section.png")
pb_subsection = pixbuf_new_from_file(Installation().path + "/pixmaps/tree_subsubsection.png")
pb_subsubsection = pixbuf_new_from_file(Installation().path + "/pixmaps/tree_subsubsection.png")
pb_paragraph = pixbuf_new_from_file(Installation().path + "/pixmaps/tree_paragraph.png")
pb_subparagraph = pixbuf_new_from_file(Installation().path + "/pixmaps/tree_paragraph.png")
pb_include = pixbuf_new_from_file(Installation().path + "/pixmaps/tree_include.png")
pb_includegraphics = pixbuf_new_from_file(Installation().path + "/pixmaps/tree_includegraphics.png")

pb_preamble = pixbuf_new_from_file(Installation().path + "/pixmaps/preamble.png")
pb_package = pixbuf_new_from_file(Installation().path + "/pixmaps/package.png")
pb_newcommand = pixbuf_new_from_file(Installation().path + "/pixmaps/run.png")
pb_document = pixbuf_new_from_file(Installation().path + "/pixmaps/document.png")


class BibItem(object):
	"""
	Container for 'simple' BibTeX items in a 'thebibliography' environment
	"""
	def __init__(self, key):
		self._key = key
	
	@property
	def key(self):
		return self._key
	
	@property
	def markup(self):
		return ""


class LatexParser(object):
	"""
	A simple LaTeX parser
	"""
	#
	# TODO: we should not clear and rebuild the outline tree
	# on each parse but create a temporary tree and apply some
	# kind of diff to the exported tree. We could preserve
	# the selection state of the TreeView.
	#
	
	def __init__(self):
		#
		# TODO: include \begin{thebibliography} in outline
		#
		self.__p_comment = compile("%.*$", MULTILINE)
		
		self.__p = compile(r"""
				\\ (?P<command>part
					|chapter
					|section
					|subsection
					|subsubsection
					|paragraph
					|subparagraph
					|include
					|includegraphics
					|input) 
					\*? 
					(\[.*?\])?
					\{""", VERBOSE)
		
		self.__p_label = compile(r"\\label\{(?P<name>.*?)\}")
		self.__p_bibitem = compile(r"\\bibitem(\[.*?\])?\{(?P<name>.*?)\}")
		#
		# TODO: improve this regex
		#
		self.__p_environment = compile(r"\\(?P<command>begin|end)\{(?P<name>[^\}]+)\}")
		self.__p_newcommand = compile(r"\\newcommand\{\\(?P<name>\w*)\}(\[(?P<args>[1-9]?)\])?\{")
		self.__p_package = compile(r"\\usepackage(\[\w*\])?\{(?P<name>\w*)\}")
		self.__p_bibliography = compile(r"\\bibliography{")
		
		self.__outline_model = TreeStore(str, Pixbuf, object)
		self.__labels = []
		self.__bibitems = []
		self.__environment_stack = []
		self.__newcommands = []
	
	def parse(self, string, offset, outline=True):
		
		# remove comments
		for match in self.__p_comment.finditer(string):
			start = match.start(0)
			end = match.end(0)
			
			string = "%s%s%s" % (string[:start], (end - start)*" ", string[end:])
			
		# TODO: remove verbatims and listings
		
		# split input string into preamble and document
		preamble, document, document_offset = self.__split(string)
		
		# parse
		if outline:
			self.__outline_model.clear()
			
			if preamble:
				self.__parse_preamble(preamble)
				
			self.__parse_document(document, document_offset)
		
		#
		# labels
		#
		self.__labels = []
		for match in self.__p_label.finditer(document):
			self.__labels.append(match.group("name"))
		
		#
		# bibitems
		#
		# TODO: find thebibliography environment and search it
		self.__bibitems = []
		for match in self.__p_bibitem.finditer(document):
			self.__bibitems.append(BibItem(match.group("name")))
		
		#
		# bib files
		#
		#
		# TODO: strip filenames etc
		#
		self.__bibliography_files = []
		for match in self.__p_bibliography.finditer(string):
			start = match.end(0) - 1
			try:
				end = self.__match_braces(string, start)
				s = string[start + 1 : end]
				if len(s):
					self.__bibliography_files = s.split(",")
			except IndexError:
				error(self, "parse", "Malformed document")
			break
		
		#
		# environments
		#
		self.__environment_stack = []
		
		for match in self.__p_environment.finditer(string[:offset]):
			if match.group("command") == "begin":
				self.__environment_stack.append(match.group("name"))
			elif match.group("command") == "end":
				try:
					e = self.__environment_stack.pop()
					if e != match.group("name"):
						error(self, "parse", "Parse error: unexpected \\end{%s}" % match.group("name"))
						break
				except IndexError:		# pop from empty list
					error(self, "parse", "Parse error: unexpected \\end{%s}" % match.group("name"))
					break
	
	def __split(self, string):
		"""
		"""
		sep = "\\begin{document}"
		parts = string.split(sep)
		
		if len(parts) == 1:
			debug(self, "__split", "No preamble found")
			return (None, string, 0)
		elif len(parts) == 2:
			#log(self, "__split : <<<%s>>>, <<<%s>>>, <<<%s>>>" % (parts[0], parts[1], len(parts[0])))
			return (parts[0], parts[1], len(parts[0]) + len(sep))
		else:
			debug(self, "__split", "Error: multiple \\begin{document}")
			return (None, string, 0)
	
	def __parse_preamble(self, preamble):
		"""
		"""
		preamble_node = self.__outline_model.append(None, ["Preamble", pb_preamble, None])
		
		#
		# packages
		#
		for match in self.__p_package.finditer(preamble):
			name = match.group("name")
			start = match.start(0)
			end = match.end(0)
			
			self.__outline_model.append(preamble_node, [name, pb_package, OutlineNodeRange(start, end)])
		
		#
		# newcommands
		#
		self.__newcommands = []
		
		for match in self.__p_newcommand.finditer(preamble):
			name = match.group("name")
			args = match.group("args")
			start = match.start(0)
			end = match.end(0)
			
			self.__outline_model.append(preamble_node, [name, pb_newcommand, OutlineNodeRange(start, end)])
			
			# generate Command object
			#
			command = Command(name, None)
			
			variant = Variant(command)
			command.variants.append(variant)
			
			if args:
				for n in range(int(args)):
					variant.arguments.append(MandatoryArgument("#%s" % (n + 1)))
			
			self.__newcommands.append(command)
		
	def __parse_document(self, document, offset):
		"""
		"""
		
		document_node = self.__outline_model.append(None, ["Document", pb_document, None])
		
		lastAbsLevel = 0			## last absolute level of element
		treeOffset = 0				## current offset in the tree
		parentNodes = [document_node]
		
		for match in self.__p.finditer(document):
			command = match.group("command")
			
			# argument
			
			#
			# TODO: skip optional argument with __match_braces and not in the regex
			# 
			arg_start = match.end(0) - 1
			try:
				arg_end = self.__match_braces(document, arg_start)
			except IndexError:
				debug(self, "__parse_document", "Parse error")
			
			argument = document[arg_start + 1 : arg_end]
			start = match.start("command") - 1 + offset
			end = arg_end + 1 + offset
			
			try:
				#
				# structure command
				#
				absLevel = levels[command]
				
				delta = absLevel - lastAbsLevel
				lastAbsLevel = absLevel
		
				if delta >= 1:
					treeOffset += 1
				else:
					treeOffset += delta
					if treeOffset < 1:
						treeOffset = 1
						
				try:
					pixbuf = {"part" : pb_part, "chapter" : pb_chapter, "section" : pb_section,
							"subsection" : pb_subsection, "subsubsection" : pb_subsubsection,
							"paragraph" : pb_paragraph, "subparagraph" : pb_subparagraph}[command]
				except KeyError:
					pixbuf = None
				
				element = OutlineNodeRange(start, end)
				
				node = self.__outline_model.append(parentNodes[treeOffset - 1], 
												[argument, pixbuf, element])
				
				try:
					parentNodes[treeOffset] = node
				except IndexError:
					parentNodes.append(node)
					
			except KeyError:
				#
				# no structure command
				#
				argument = basename(argument)
				
				pixbuf = None
				
				if command == "include" or command == "input":
					pixbuf = pb_include
					element = FileReference(start, end, argument)
				elif command == "includegraphics":
					pixbuf = pb_includegraphics
					element = OutlineNodeRange(start, end)
				
				node = self.__outline_model.append(parentNodes[treeOffset], 
												["<i>%s</i>" % argument, pixbuf, element])
			
	def __match_braces(self, string, start):
		c_open = string[start]
		if c_open == "{":
			c_close = "}"
		elif c_open == "[":
			c_close = "]"
		else:
			error(self, "__match_braces", "<start> must point to the opening bracket, it points to '%s' in '%s'" % (string[start], string))
			return start
		
		k = 0
		i = start
		while True:
			i += 1
			
			if string[i] == c_open:
				k += 1
			elif string[i] == c_close:
				k -= 1
			
			if k < 0:
				return i
	
	@property
	def outline_model(self):
		return self.__outline_model
	
	@property
	def bibliography_files(self):
		return self.__bibliography_files
	
	@property
	def bibitems(self):
		return self.__bibitems
	
	@property
	def labels(self):
		return self.__labels
	
	@property
	def open_environment(self):
		try:
			return self.__environment_stack[-1]
		except IndexError:
			return None
	
	@property
	def newcommands(self):
		return self.__newcommands
	