# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from gtk import combo_box_new_text, Dialog, DIALOG_MODAL, DIALOG_DESTROY_WITH_PARENT, STOCK_CANCEL, \
		RESPONSE_CANCEL, STOCK_OK, RESPONSE_OK, Label, Table, FileChooserDialog, FILE_CHOOSER_ACTION_OPEN, \
		STOCK_OPEN, RESPONSE_ACCEPT, FileChooserButton, RadioButton, SpinButton, Adjustment

class ListingDialog(Dialog):
	def __init__(self, parent, languages):
		Dialog.__init__(self, "Insert Listing", parent, DIALOG_MODAL | DIALOG_DESTROY_WITH_PARENT, \
					(STOCK_CANCEL, RESPONSE_CANCEL, STOCK_OK, RESPONSE_OK))
		self.set_border_width(5)
		self.set_has_separator(False)
		self.set_resizable(False)
		self.vbox.set_spacing(5)
		
		self.__languages = languages
		
		# content
		lbl_content = Label("<b>Content</b>")
		lbl_content.set_use_markup(True)
		lbl_content.set_alignment(0, .5)
		
		self.__rad_environment = RadioButton(None, "Environment")
		
		self.__rad_file = RadioButton(self.__rad_environment, "File:")
		self.__rad_file.connect("toggled", self.__file_toggled)
		
		self.__dlg_file = FileChooserDialog(action=FILE_CHOOSER_ACTION_OPEN, 
										buttons=(STOCK_OPEN, RESPONSE_ACCEPT, STOCK_CANCEL, RESPONSE_CANCEL))
		self.__dlg_file.set_local_only(True)
		
		self.__btn_file = FileChooserButton(self.__dlg_file)
		self.__btn_file.set_sensitive(False)
		
		# language
		lbl_style = Label("<b>Style</b>")
		lbl_style.set_use_markup(True)
		lbl_style.set_alignment(0, .5)
		
		self.__cmb_language = combo_box_new_text()
		self.__cmb_language.append_text("(None)")
		for language in self.__languages:
			self.__cmb_language.append_text(language.name)
		self.__cmb_language.set_active(0)
		
		lbl_language = Label("Language:")
		lbl_language.set_alignment(0, .5)
		
		self.__cmb_dialect = combo_box_new_text()
		
		self.__lbl_dialect = Label("Dialect:")
		self.__lbl_dialect.set_alignment(0, .5)
		
		self.__lbl_dialect.set_sensitive(False)
		self.__cmb_dialect.set_sensitive(False)
		
		# tab size
		self.__spn_tabsize = SpinButton(Adjustment(4, 0, 100, 1))
		self.__spn_tabsize.set_activates_default(True)
		
		lbl_tabsize = Label("_Tab Size:")
		lbl_tabsize.set_alignment(0, .5)
		lbl_tabsize.set_use_underline(True)
		lbl_tabsize.set_mnemonic_widget(self.__spn_tabsize)
		
		# pack
		table = Table()
		
		table.set_border_width(5)
		table.set_row_spacings(5)
		table.set_col_spacings(5)
		
		table.attach(lbl_content, 0, 5, 0, 1)
		table.attach(self.__rad_environment, 1, 5, 1, 2)
		table.attach(self.__rad_file, 1, 2, 2, 3)
		table.attach(self.__btn_file, 2, 5, 2, 3)
		table.attach(lbl_style, 0, 5, 3, 4)
		table.attach(lbl_language, 1, 2, 4, 5)
		table.attach(self.__cmb_language, 2, 3, 4, 5)
		table.attach(self.__lbl_dialect, 3, 4, 4, 5)
		table.attach(self.__cmb_dialect, 4, 5, 4, 5)
		table.attach(lbl_tabsize, 1, 2, 5, 6)
		table.attach(self.__spn_tabsize, 2, 3, 5, 6)
		
		table.set_col_spacing(0, 15)
		table.set_row_spacing(2, 15)
		
		self.vbox.add(table)
		
		self.vbox.show_all()
		
		self.__cmb_language.connect("changed", self.__language_changed)
	
	def __language_changed(self, combobox):
		i = self.__cmb_language.get_active()
		if (i == 0):
			self.__cmb_dialect.get_model().clear()
			self.__lbl_dialect.set_sensitive(False)
			self.__cmb_dialect.set_sensitive(False)
			return
		else:
			l = self.__languages[i - 1]
		
		self.__cmb_dialect.get_model().clear()
		
		if len(l.dialects):
			self.__lbl_dialect.set_sensitive(True)
			self.__cmb_dialect.set_sensitive(True)
		else:
			self.__lbl_dialect.set_sensitive(False)
			self.__cmb_dialect.set_sensitive(False)
			return

		i = 0
		for dialect in l.dialects:
			if dialect.name:
				self.__cmb_dialect.append_text(dialect.name)
			else:
				self.__cmb_dialect.append_text("(None)")
			
			if dialect.default:
				self.__cmb_dialect.set_active(i)
			i += 1
	
	def __file_toggled(self, checkbutton):
		self.__btn_file.set_sensitive(self.__rad_file.get_active())
	
	@property
	def filename(self):
		if self.__rad_file.get_active():
			return self.__dlg_file.get_filename()
		else:
			return None
	
	@property
	def language(self):
		i = self.__cmb_language.get_active()
		if i == 0:
			return None
		else:
			return self.__languages[i - 1].name

	@property
	def dialect(self):
		i = self.__cmb_language.get_active()
		if i == 0:
			return None
		
		l = self.__languages[i - 1]
		j = self.__cmb_dialect.get_active()
		try:
			return l.dialects[j].name
		except IndexError:
			return None
	
	@property
	def tabsize(self):
		return self.__spn_tabsize.get_value_as_int()
	